use rhai::{FnNamespace, FuncRegistration, ImmutableString, Module};
use std::{cell::RefCell, rc::Rc};

use crate::ui::UiState;

pub fn build_ui_module(state_src: Rc<RefCell<UiState>>) -> Module {
	let mut module = Module::new();
	module.set_id("GalacticaUiModule");

	let state = state_src.clone();
	let _ = FuncRegistration::new("go_to_scene")
		.with_namespace(FnNamespace::Internal)
		.set_into_module(&mut module, move |scene: ImmutableString| {
			let mut ui_state = state.borrow_mut();
			ui_state.set_scene(scene);
		});

	let state = state_src.clone();
	let _ = FuncRegistration::new("get_camera_zoom")
		.with_namespace(FnNamespace::Internal)
		.set_into_module(&mut module, move || state.borrow().camera.get_zoom());

	let state = state_src.clone();
	let _ = FuncRegistration::new("set_camera_zoom")
		.with_namespace(FnNamespace::Internal)
		.set_into_module(&mut module, move |z: f32| {
			state.borrow_mut().camera.set_zoom(z)
		});

	return module;
}
