use galactica_util::{clockwise_angle, to_degrees, to_radians};
use nalgebra::{Rotation2, Vector2};
use rhai::{CustomType, TypeBuilder};

#[derive(Debug, Clone)]
pub struct UiVector {
	val: Vector2<f32>,
}

impl UiVector {
	pub fn new(x: f32, y: f32) -> Self {
		Self {
			val: Vector2::new(x, y),
		}
	}

	pub fn rotate(&mut self, angle: f32) -> Self {
		return UiVector {
			val: Rotation2::new(to_radians(angle)) * self.val,
		};
	}

	pub fn clockwise_angle(&mut self, other: UiVector) -> f32 {
		return to_degrees(clockwise_angle(&self.val, &other.val));
	}
}

impl CustomType for UiVector {
	fn build(mut builder: TypeBuilder<Self>) {
		builder
			.with_name("Vector")
			.with_fn("Vector", Self::new)
			.with_fn("rotate", Self::rotate)
			.with_fn("clockwise_angle", Self::clockwise_angle)
			.with_fn("+", |a: UiVector, b: UiVector| UiVector {
				val: a.val + b.val,
			})
			.with_fn("-", |a: UiVector, b: UiVector| UiVector {
				val: a.val - b.val,
			})
			.with_fn("/", |s: &mut Self, x: f32| UiVector { val: s.val / x })
			.with_fn("*", |s: &mut Self, x: f32| UiVector { val: s.val * x })
			.with_fn("x", |s: &mut Self| s.val.x)
			.with_fn("y", |s: &mut Self| s.val.y)
			.with_fn("norm", |s: &mut Self| s.val.magnitude());
	}
}
