use nalgebra::Vector2;
use rhai::{Dynamic, ImmutableString};

use super::super::api::Rect;
use crate::{InputEvent, RenderInput, RenderState};

#[derive(Debug, Clone)]
pub struct UiScrollbox {
	pub name: ImmutableString,
	pub rect: Rect,
	pub offset: Vector2<f32>,

	has_mouse: bool,
}

impl UiScrollbox {
	pub fn new(name: ImmutableString, rect: Rect) -> Self {
		Self {
			name,
			rect,
			offset: Vector2::new(0.0, 0.0),
			has_mouse: false,
		}
	}

	pub fn step(&mut self, _t: f32) {
		// TODO: inertia
	}

	pub fn handle_event(
		&mut self,
		input: &RenderInput,
		state: &mut RenderState,
		event: &InputEvent,
	) -> Option<Dynamic> {
		self.handle_event_with_offset(input, state, event, Vector2::new(0.0, 0.0))
	}

	pub fn handle_event_with_offset(
		&mut self,
		input: &RenderInput,
		state: &mut RenderState,
		event: &InputEvent,
		offset: Vector2<f32>,
	) -> Option<Dynamic> {
		let mut r = self
			.rect
			.to_centered(&state.window, input.ct.config.ui_scale);
		r.pos += offset;

		match event {
			InputEvent::MouseMove(pos) => {
				if r.contains_mouse(state, pos) && !self.has_mouse {
					self.has_mouse = true;
				}

				if !r.contains_mouse(state, pos) && self.has_mouse {
					self.has_mouse = false;
				}
			}

			InputEvent::Scroll(x) => {
				if self.has_mouse {
					self.offset.y -= x;
				}
			}

			_ => return None,
		}

		return None;
	}
}
